	 #####		SCRIPTS FOR PART 5

## The following packages should be installed for Part 5
# lme4, languageR, rms, mlogit. The Mass package should have been already 
# installed for use with script 4; if not, install it now.

# NOTE: rms supercedes the Design package

#####    		Session 26: Mixed-Effects Models

# The mixture of fixed and random factors in a study presents
# problems for the standard ANOVA; exact F tests of some terms of
# interest are lacking. The methods exemplified in Sessions 15 and 16
# (e.g., quasi-F and minF tests) typify applications in the published
# literature. However, Mixed-effects models have greater generality, and often
# greater power because they use all the data without the need to average 
# over subjects or items, or to replace missing scores, and also because
# their use of maximum likeihood estimationn provides better estimates.
# Furthermore, they enable inclusion of all factors that may affect the scores.

# In this session, we analyze mixed-effects data using functions from
# the lme4 and languageR packages. 

### Sources Introducing the Application of R to Mixed Modeling
# Bates provides an introduction to the lmer function at 
# http://cran.r-project.org/doc/Rnews/Rnews_2005-1.pdf
# Select "Fitting linear mixed models in R". 

# Another source, particularly useful with psycholinguistics data is
# "Analyzing Linguistic Data:A Practical Introduction to Statistics using R" 
# by Harald Baayen. The book provides an introduction to R, and Chapter 7
# covers mixed-effects modeling.

# An excellent article in the Journal of Memory and Language is:
# "Mixed-effects modeling with crossed random effects for 
#  subjects and items" by Baayen, Davidson, and Bates (2007, pp. 390-412).
#  Scott's powerpoint provides a useful summary of this article; 
#  go to https://netfiles.uiuc.edu/brownsch/www/Baayen08highlights.ppt.

#  Also see the lme4 and languageR package descriptions on the internet.
#  Googling "R mixed effects models" provides many more references.

#  Assuming you have installed lme4, load it:
     library("lme4") 

# Next we illustrate some of the functions in the package. We employ data
# from a mixed design in which subjects are presented with 15 film clips, 
# 5 for each of three durations. Twelve subjects are asked for an immediate
# {I} rating of the unpleasantness of the film clips and 12 rate after a 
# delay (D). 

### Analyzing Data from a Mixed Design
# Read in the file "film clips.csv":
     rating.data = read.csv(file.choose(),header=T,sep=",")
 	head(rating.data)
	attach(rating.data)
	del.f=factor(Delay)
	dur.f=factor(Duration)

# del.f and dur.f are fixed-effect factors; Subject and Item have random
# effects. del.f is between subjects, and dur.f and Item are within 
# subjects. There are 24 subjects, 12 in each of two delays; and there are 
# 15 items, five in each of three durations.

# The  del.f x dur.f means are given by
    tapply(Y,list(del.f,dur.f),mean,data=rating.data)

## Exercise 26.1
# Using R code, find the del.f and dur.f marginal means and standard errors.

## A Model with Random Slopes
# We first test the effects of the two factors, employing a model
# that includes all possible sources of variance. The main and interaction
# effects of del.f and dur.f are included in the model. In deciding
# on the random effects that belong, we note that each subject has a score
# at all three durations. Each of these subject functions has an intercept
# and a slope. These intercepts and slopes may vary across subjects.
# Those variances will be included in the model. The slope variance
# implies a Subjects x dur.f interaction, as in the standard anova.
# Items cross with del.f so that when the item's scores are plotted against
# del.f, we have an intercept and a slope; these variances will also
# be included in the model. These considerations lead us to the following 
# model.

      rating.data.lmer1 = lmer(Y~del.f*dur.f+
	    (1+dur.f|Subject)+(1+del.f|Item))

## Exercise 26.2
# Note the difference in how we represent subjects and items in the model.
# What in the experimental design necessitates this difference? For example,
# why is there no (1+del.f|Subject) term?

# To complete the analysis,
      rating.data.lmer1

## Interpreting the output
# The first statement of interest is that the model was fit by REML.
# This is a restricted maximum likelihood method that often
# provides better estimates of variance components than the standard ML.
# Next is the formula we used. The model states that Y is a 
# function of del.f, dur.f, and their interaction. These are the fixed
# effects. For the random effects, we have (1+del.f}Item) and
# (1+dur.f|Subject). Both Subject and Item intercepts (designated
# by 1) vary, as do the item slopes (plotted aainst test levels) and subject
# slopes (plotted against duration levels).

# Next are various measures of goodness of fit.  The smaller AIC and BIC are, 
# the better the fit; the larger (less negative) log likelhood is, 
# the better the fit. AIC and BIC are functions of log-likelihood, adjusted
# for the number of parameters estimated. They are useful in comparing models.

# Estimates of the variances of various random effects are next and include
# effects contributed by the subject intercepts and subject effects
# at the second and third durations, as well as variances of the item effects 
# contributed by the intercept and  the second test. We also can obtain
# the effects of individual subjects and items; e.g., for the 1st 
# 6 subjects:

     head(ranef(rating.data.lmer1)$Subject)

# Following the random effects are the estimates of the fixed effects. 
# The  intercept contributes to all scores. The dur.f and test
# estimates are estimates of differences from their bases, The interaction 
# terms are estimates of del.f x dur.f effects.

# The last part of the output of the lmer function is the estimated 
# correlations of the fixed effects. These correlations reflect that
# the design involves repeated measures.

## Comparing Models
# The t's for dur.f seem to be significant but the df are not readily 
# estimable. However, we can test the dur.f effect more directly
# by formulating a model without dur.f fixed effects, then comparing the 
# models with and without dur.f. The question is whether the more inclusive
# model provides a significantly better fit to the data. The reduced morel is

	rating.data.lmer2 = lmer(Y~del.f+(1+dur.f|Subject)
	    +(1+del.f|Item))

# And the output:
	rating.data.lmer2

# Now we can compare the two models:
	anova(rating.data.lmer1,rating.data.lmer2)
# Model 1 has a higher log likelihood and lower  BIC and AIC values than 
# Model 2, indicating an advantage for the more inclusive Model 1.
# The chi square p value is essentially 0. We conclude that dur.f
# has a significant effect on the scores. 

## Models with Only Random Intercepts
# Although dur.f appears to be an important contributor to the model,it
# is possible that the random slopes are not necessary. We next consider a
# model that does not assume that subject variance depends on dur.f
# or item variance on the test.
	rating.data.lmer3 = lmer(Y~del.f*dur.f+(1|Subject)+(1|Item))
	rating.data.lmer3
# The output now contains only intercept values for Subject and Item.
# Again, only duration effects appear to be significant.

# We can compare this model with the random slopes model, Model 1:
	anova(rating.data.lmer1,rating.data.lmer3)
# The p value indicates that there is no significant difference between
# the fits of the two models to the data. The assumption that the fixed
# effects depend on subjects or items seems unnecessary.

# The random intercepts model leads to the same conclusions as Model 1
# (random slopes).It is simpler and, in the interests of parsimony,
# we may prefer to work with it in subsequent analyses of the data; e.g.,
# in tests of contrasts, or in examining random effects of individual
# subjects or items.  Also, as we shall see next, it provides an 
# additional way to judge statistical significance, a way not available
# with random slopes models.


### p Values and HPDs (Bayesian Hierarchical Posterior Density Bounds)
# We use Baayen's pvals.fnc to obtain the p value through a simulation based 
# on the data set using the MCMC (Markov Chain Monte Carlo)
# method. Because the default is 10,000 (nsim=10000), the computations 
# may take several minutes. The simulation also yields lower and upper
# bounds (on the distibutions of fixed effects. If not installed, do so. Then.
	library(languageR)

#  Proceeding with the rating.data example,
     pvals.fnc(rating.data.lmer3) 
	
# This does the calculations and plots the estimated distributions
# of the effects. There is some indication of non-normality, but not sufficient 
# to be troublesome. If you wish to omit the plot, add "addPlot=FALSE" to
# the preceding command.
	dev.off()

# The fixed and random estimates are those seen previously for 
# rating.data.lmer3. In addition, we have estimates that are based on
# the simulation, MCMCmean and MCMCmedian. For this data set, the simulation
# results are quite clse to the original estimates. This is also true
# of the simulated p values (pMCMC) and Pr(>|t|) assuming df =
# number of observations - number of parameters; this p value is 
# usually liberal and lower than the MCMC result. 

# The HPD values, hierarchical posterior densitiy bounds, are like confidence  
# interval bounds. However, a major difference is that within the
# Bayesian framework, the intercept or effect is viewed as a random
# variable so that the bounds are values ccntaining  -- e.g. --
# 95% of the distributionn of the parameter. With confidence
# intervals, we assume a fixed parameter and the 95% bounds refer to the
# proportion of samples that contain the "true" parameter value. 

## NOTE: The pvals.fnc function is not applicable to the random slopes model.
# For example, pvals.fnc(rating.data.lmer1) yields the error message:
# "   MCMC sampling is not implemented in recent versions of lme4
# for models with random correlation parameters"

## Exercise 26.3
# Read "SAB_Long.cvs"which is a long form of a subjects x A x B data set. 
# Assume B is a random effects variable,perhaps items, and A has fixed 
# effects and is the treatment factor. Analyze the random slopes model.
# Then analyze possible simpler models and carry out model comparisons.
# State your conclusions.

## Exercise 26.4
# In rating.data, items were nested within durations. In "film clips_b.csv" 
# the same 5 items are in each duration (crossing instead of nesting). Read
# the file and carry out the analyses. (Items are still assumed
#  to be randomly sampled but now items and duration cross.)

## Exercise 26.5
# In the preceding example, we use a polynomial contrast. Redo the
# analysis with treatment (dummy or index coding). How do the estimates
# compare with those obtained when we analyzed rating.data.lmer3 without
# specifying contrasts?


### Summing Up the Mixed-Models Approach
# The mixed-effects  approach has several advantages relative to
# approaches such as F1, F2, and quasi_F's:
# More powerful than the alternatives
# Robust in the presence of missing data; no need to drop cases
# No need for averaging as with tests against item or subject variability
# Can explicitly model subject and item effects on both intercepts
# and slopes
# Can explicitly test which random effects should be included in the model
# Can use likelihood ratio tests to compare models
# The approach focuses on two questions we tend to overlook when
# we emphasize p values:
#   (1) What are the sizes of effects and variance components?
#   (2) Which model is best? 


### Contrasts in Mixed-Effects Models
# Assuming that the three durations are equally spaced on a time continuum,
# we might wish to do a trend analysis using the contr.poly function. Define
     K = contr.poly(3)
# Then, add the contrasts to the rating.data.lmer3 model
	rating.data.lmer4 = lmer(Y~del.f*dur.f+(1|Subject)+
	   (1|Item),contrasts=list(dur.f= K),data=rating.data)
        rating.data.lmer4
# The linear component of the dur.f factor (dur.f.L) appears to be
# significant.

## To test for significance and obtain bounds on the effects
      pval.lmer4=pvals.fnc(rating.data.lmer4,addPlot=F)
      pval.lmer4$fixed
# The slope of the average duration function (dur.f.L)
# differs significantly from zero and no other terms are significant.

## Exercise 26.6
# Using Model 3, incorporate the contr.treament contrast. How do the fixed
# results table compare with those from rating.data.lmer3? What does this say
# about the default contrasts in Model 3?

# clean up
	detach(rating.data)
	rm(del.f,dur.f,rating.data,rating.data.lmer1,rating.data.lmer2,rating.data.lmer3,
		rating.data.lmer4,pval.lmer4,K)

### A Latin Square Design
# The "LatSq_Data_3.csv" file contains data for 24 subjects. Each subject is tested 
# on three lists, with the list items presented for one of three
# durations. The assignment of lists to durations is counterbalanced to
# form a 3 x 3 Latin square, 8 subjects in each row, 4 of whom are tested at 
# one delay, and 4 at another. The score, Y, is the percent of items recalled.
# To read the "LatSq_Data_3.csv" file,
     lsq.data = read.csv(file.choose(),sep=",", header=T)
     head(lsq.data)

# Now to analyze the data, beginning with a model that includes all factors.
# Make sure the lme4 and languageR packages are loaded.
# Note: Rows refers to the three rows of the Latin Square.
 
## Model Comparisons
# We first evaluate this model
      lsq.lmer=lmer(Y~Delay*Duration+(1|Subject)+(1|Row)
         +(1+Duration|List),data=lsq.data)
     summary(lsq.lmer)
# The Duration effects are clearly significant, and there is a suggestion
# of a Delay*Duration interaction.

# Because the estimate of the Row contribution to the intercept variability
# is essentially zero, we can delete Row from the model.
     lsq.lmer2=lmer(Y~Delay*Duration+(1|Subject)
         +(1+Duration|List),data=lsq.data)
     summary(lsq.lmer2)

# We can compare the models:
	anova(lsq.lmer,lsq.lmer2)
# There is no indication that Model 1 provides a better fit.

# Given that there is little variability due to lists (compared to the
# residual variance), we try one more model:
    lsq.lmer3=lmer(Y~Delay*Duration+(1|Subject), data=lsq.data)
     summary(lsq.lmer3)
	
# Comparing models,
	anova(lsq.lmer2,lsq.lmer3)
# The models do not differ significantly, though the AIC and BIC statistics
# favor Model 3.

## To obtain the phd bounds and p values,
     pvals.lsq = pvals.fnc(lsq.lmer3,addPlot=F)
     pvals.lsq$fixed
# It is clear that the intercept differs from zero, and that the dur2
# and dur3 means differ from the intercept. To further examine the data,
	anova(lsq.lmer3)
# Only the duration man effect is significant.

## Trend Analysis for the Latin Square    
    lsq.lmer4=lmer(Y~Delay*Duration+(1|Subject),
     	  contrasts=list(Duration="contr.poly"),data=lsq.data)
     summary(lsq.lmer4)
# And for p values and obunds on estimates,
	pvals.fnc(lsq.lmer4,addPlot=F)

# Only the linear component of Duration is significant. Note that the
# simulation p values (MCMC) are, as is typical, higher than those
# assuming assymptotic df for the t distribution, with the exception
# of the results for Delay.

# Cleam up
	rm(lsq.lmer,lsq.lmer2,lsq.lmer3,lsq.lmer4,lsq.data,pvals.lsq)

## Exercise 26.7
# 24 air controllers (12 under 60 and 12 over 60, were presented with
# a series of four simulated flight situations and their response times
# were recorded. The sequence of situations (A) was counterbalanced over
# four blocks of time (B). The data set is in the "LatinSq_4.csv" file.
# Note that only Subjects and Rows have random effects.
# Use the lmer and pvals.fnc functions to evaluate the effects. Carry out a 
# polynomial analysis of the B (trials) variable.

############################################################

#####              Script for Session 27
#####	    Introduction to Logistic Regression Models

### Binary Logistic Regression: Binary Predictor
# The data we analyze first are from a study of self-injurious behavior
# (sib) as a function of several variables, three of which -- sex, age, and
# agetoin (age at which the monkey became independent of its mother) -- are
# in the file "sib_data.csv". We will use only the response (sib) and sex 
# variables here; the age and agetoin variables will appear in an exercise
# later in the session.

# Read in the data
	sib.data = read.csv(file.choose(),header=T,sep=",")
	head(sib.data)			# sex=1 ==> male, 0 ==> female
	attach(sib.data)

# Create a fequency table
	table(sib,sex)				# 0 = no sib; 1 = sib
							# 0 = female; 1 = male

## The Odds Ratio (OR)
# Let pm = P(sib=1|male) and qm = P(sib=0|male) = 1-pm. Similarly,
#	 pf = P(sib=1|female) and qf = P(sib=0|female) = 1-pf. 
# Then these conditional probabilities are (see Session 3)
	prob=round(prop.table(table(sib,sex),2),4); prob
# The odds of a male monkey having sib relative to not having it are
	odds_m=prob[2,2]/prob[1,2]; odds_m 	# = .228
# and the odds for female monkeys is 
	odds_f=prob[2,1]/prob[1,1]; odds_f		# = .025
# The odds ratio is
	OR = odds_m/odds_f; OR				#= 9.092

# The odds of a male having sib is about 9 times greater than for a
# female. If probabilities, odds, and odds ratios were all we wanted, 
# there would be no need to introduce the logistic model. However, we 
# do so in this context for several reasons. We want a framework that 
# provides us with more than just these probabilities (e.g., confidence
# limits on the OR would be useful, as would a significance test), and
# that generalizes to more complex situations (e.g., predictors that have
# more than two levels, and predictors that are continuous). Therefore, we 
# consider this design as a special case of logistic regression.

## The Logistic Regression Model
# Let Y be the dependent measure  and X be the independent variable
# (sex). Then we can construct a regression equation:

# 	E(Y|x) = b0 +b1*x; x is coded here as 0 (female) or 1 (male)

# To simplify notation, let p_x = E(Y|x). The logistic transformation yields
#	p_x = exp(b0+b1*x)/(1+exp(b0+b1*x))
# Defining the odds as p_x/q_x, substitutiong the above expression, 
# and simplifying,
#	p_x/q_x = exp(b0 + b1*x)

# R, as well as most statistical software, provides estimates of b0 and
# b1, together with significance tests and other useful information. The 
# basis for the analysis is the logit (the log of the oddds), where
#   			logit_x = b0+b1*x
# The log of the oddds ratio (the OR) is 
#		log(OR) = log[(p_1/q_1)(p_0/q_0)]
# which is the difference between logits calculated at x=1 and x=0. Therefore,
#		log(OR) = (b0+b1*1)- (b0+b1*0) = b1
# In summary, log(OR) = b1 and OR = exp(b1).

## The Analysis
# If not already installed, install the MASS package. Then
	library(MASS)
	sib.glm=glm(sib~sex,family=binomial,data=sib.data)
	summary(sib.glm)

# The summary first presents the model. Next is a five-point summary of
# the Deviance Residuals; these residuals measure how well the model 
# fits the observed data, The coefficients, intercept and sex, are
# b0 and b1, respectively. The p values rest on the assumption of an 
# asymptotic normal distribution of z, and therefore may be misleading for
# small N. In this example, both coefficients differ significantly from zero. 

# The log(OR), using the odds ratio calculated from the tabled frequencies, is
	log(OR)		# = 2.207
# Note that this equals the sex coefficient (2.206) within rounding error. 
# In general, with a single predictor and a binary response, the b1
# coefficient will equal the odds ratio. Therefore, the significant sex
# coefficient indicates significantly higher odds of sib for male than 
# for female monkeys.

# Deviance is -2*log(likelihood; the null deviance is the logL for a
# model when only b0 is estimated, and therefore df_null = N -1, or 361.
# The residual deviance reflects the fit of sib.glm, and so 2 df are lost
# in estimating b0 and b1. Our z test of sex (i.e., of b1) indicates its
# inclusion improves the fit. Alternatively, we can compare the two 
# deviance values:
	pchisq(255.71-228.71,1,lower.tail=F)	 # null-residual deviance
			 					      # p = 2.034555e-07
# The inclusion of sex in the model significantly reduces the deviance.

## More on the Odds Ratio
# As noted, log(OR)=b1. Therefore, OR equals
	exp(sib.glm$coefficients[2])	# slightly > 9, as obtained earlier
							# from the prob table 
# We can readily obtain confidence intervals for log(OR):
	confint(sib.glm)
# The limits on the sex coefficient (b1) are limits on log(OR). To get
# linits on OR,
	exp(confint(sib.glm))

## A Note on Coding
# The default coding of binary variables such as sex is 0,1. Suppose
# we change the sex coding to -1,1:
	sex2 = -1+2*sib.data$sex		# sex is now coded -1 and 1
	sib2.glm=glm(sib~sex2,family=binomial,data=sib.data)
	summary(sib2.glm)
# Note the change in the sex coefficient; with the 0/1 coding:
	sib.glm$coefficients[2]
# whereas with the -1/1 coding:
	sib2.glm$coefficients[2]	
# the sib2 sex coefficient is 1/2 that of the sib coefficient.
# There is also an effect on the OR. The -1/1 coding will lead to a wrong
# estimate unless we are very careful. Note the OR with this coding
# is not the correct OR (@ 9.0) obtained earlier:
	exp(sib2.glm$coefficients[2])

## Exercise 27.1
# Suppose we had a predictor with levels 5 and 8; i.e.  
#	sex3 = 5+3*sib.data$sex-- perhaps number of days after independence
# from the mother. Carry out the logistic regression. What is the 
# relationship between b1 in this analysis and b1 with 0/1 coding? 
# What general principle does this suggest?

# Clean up the workspace
	detach(sib.data)
	rm(odds_f,odds_m,OR,prob,sex2,sib.glm,sib2.glm)

# Note that we retain sib.data for use in a later exercise.

### Binary Logistic Regression: Multilevel and Continuous Predictors
# Read the file "BinaryData.csv". There are 400 cases, varying in age,income,
# and political party. The dependent variable is approve (=1) or disapprove
# (=0) of the performance of a public official.
   	bin.data=read.csv(file.choose(),header=T,sep=",") #binary response
	attach(bin.data)
	head(bin.data)
# "approve" (1 = "yes") is the dependent measure; "income" and "age" will be 
# considered continuous; "party" (i.e., political party) has four levels;
# predictors with more than two levels are often referred to as 
# polychotomous or polytomous.

## Some Summary Statistics
	summary(income)		# in thousands of dollars
	summary(age)
	c(sd(income),sd(age))
	table(party)
	table(approve) 
	table(approve,party)  	# Note that p(approve=1)/p(approve=0),
						# the odds for approval, decrease as
						# we go from party A to party D.

## Exercise 27.2
# Using the appropriate R command, convert the last table to conditional 
# probabilities; e.g., P(approve = 1|party=A). 


## A Polychotomous (multilevel) Predictor
# We first ignore age and income and analyze the response only as a 
# function of the respondents' (political) party. Although we can
# use other coding, R's default is to construct three (in general,
# K-1; K is the number of levels of the predictor) dummy variables:
#	A 0 0 0
#	B 1 0 0
#	C 0 1 0
#	D 0 0 1
# The log odds (logit) for A equals b0+b1*0+b2*0+b3*0 = b0.
# Similarly, logit_B = b0+b1; logit_C = b0+b2; logit_D = b0+b3;
# Then the log of the odds ratio (OR) can be expressed in terms of 
# the regression coefficients. For example, log(OR(B,A)=logit_B-logit_A
# = b1.

# To obtain estimates of the coefficients and their SEs:
	bin.glm=glm(approve~factor(party),family=binomial)
	summary(bin.glm)

# The coefficients are the intercept (b0), b1, b2, and b3. Because b1
# equals log(OR(B,A), we conclude that being in party B as opposed to party A 
# decreases the log odds of approval. The decrease in log odds relative 
# to A is even greater  for parties C and D. This is  consistent with 
# the table of frequencies (calculated earlier) in which it is clear 
# that the ratio of odds of approval to disapproval decreases as we go from
# A to D (i.e., 33/28, 54/97, 28/93, 12/55). Furthermore, all three
# slope coefficients differ significantly from zero indicating that the ORs
# differ significantly from 1. 

## Exercise 27.3
# From the tabled frequencies (calculated previously) find the
# three odds ratios: OR(B,A), OR(C,A), OR(D,A). Then take the log of each
# and compare to the coefficients yielded by summary(bin.glm).

### The Logistic Model with Continuous Predictors
# We next include age and income in our model. Note that, because the
# predictors are not orthogonal to each other, this will affect our
# estimates of the coefficients. The simple relationship between observed
# frequencies and odds ratios based on the model (seen in Exercise 27.3)
# will no longer hold.

# Our model is now
	bin2.glm=glm(approve~age+income+factor(party), family=binomial)
	summary(bin2.glm)
#  As before, the estimates of coefficients represent the changes in
# log odds as a function of changes in the variable. For example,
#  For every $10,000 increase in income, the log-odds of approval to 
#  disapproval increases by 0.0178. Similarly, for every year increase
#  in age, they increase by .029. 

# Next consider the estimates of party coefficients. Being in party B
# as opposed to party A decreases the log odds of approval by .675.
# Similar conclusions hold for the other parties, though consistent
# with the pattern of p-values, the decrease is greater for C and D.
# All sources contribute significantly to variation in the logits.

# The null deviance is the deviance when only the intercept is estimated, 
# and the residual deviance is the result after 4 iterations of the 
# estimation process. It is the difference in deviance between the full 
# and null models. Note that we lost only one df for estimating the intercept 
# of the null model, but 5 more for the additional parameters in the final 
# result 

## Do the Parameters Improve the Fit?
# We compare the full model with the null model:
	diff.dev = bin.glm$null.deviance-bin.glm$deviance  # deviance difference
	diff.df = bin.glm$df.null-bin.glm$df.residual 	 # df differences
	pchisq(diff.dev,diff.df,lower.tail=F)	# p value from chi square test
# The full model is significantly better than the null model.

# We can also compare bin2.glm with bin.glm: does adding income and age
# to party improve the fit?
# Note that the AIC value, that takes the number of df's into consideration,
# is smaller for bin2.glm, indicating a better fit. A test is
	anova(bin2.glm,bin.glm,test="Chisq")	
# p = .0002; inclusion of age and income significantly improves the fit.

## Confidence Interval Bounds for the Estimates:
	confint(bin2.glm)

## Odds Ratios
# Generally, we are more comfortable with odds than with log odds (logits).
# We can convert:
	exp(bin2.glm$coefficients)
# The results tell us that for every $10,000 increase in income, the odds 
# of approval versus non-approval increase by a factor of 1.02. The age
# estimate of odds has a similar interpretation. For party, the 
# interpretation is slightly different. For example, party B's approval odds
# are roughly half (.51) those of Party A. The estimate of the 
# odds ratio for party D relative to that of party A is .21.

# NOTE: the results for party from  bin2.glm have a similar trend as
# those from bin.glm. However, the numerical estimates, odds, and odds 
# ratios differ because bin2.glm introduces variables that are not
#  orthogonal to Party.

## Confidence Intervals on the Odds Ratios 
	exp(confint(bin2.glm))

## Predicted Probabilities
## Calculating Predicted Probabilities
# We look at predicted approval probabilities as a function of party
# membership while holding age and income at their means, First, set
# up the data frame:
	party = c("A","B","C","D")
	income = c(mean(bin.data$income))
	age = c(mean(bin.data$age))
	newdata1 = data.frame(income,age,party)
	newdata1
# Now to get the predictions:
	newdata1$partyP = predict(bin2.glm,newdata=newdata1,type="response")
	newdata1
## NOTE: If type="response" is omitted, default is logit (log-odds) scale.

# We might get predictions for a given party when income is varied; e.g.,
	newdata2=data.frame(income=seq(20,100,10),age=mean(bin.data$age),
		party="A")
	newdata2$incomeP=predict(bin2.glm,newdata=newdata2,type="response")
	newdata2
# The predicted probability of approval rises for members of party A
# as income rises with age held at its mean.

# Some copying and pasting, with minor changes, provides results
# for membership in parties B, C, and D.
	newdata3=data.frame(income=seq(20,100,10),age=mean(bin.data$age),
		party="B")
	newdata3$incomeP=predict(bin2.glm,newdata=newdata3,type="response")

	newdata4=data.frame(income=seq(20,100,10),age=mean(bin.data$age),
		party="C")
	newdata4$incomeP=predict(bin2.glm,newdata=newdata4,type="response")

	newdata5=data.frame(income=seq(20,100,10),age=mean(bin.data$age),
		party="D")
	newdata5$incomeP=predict(bin2.glm,newdata=newdata5,type="response")

	predictedP = data.frame(round(cbind(newdata2[,4],newdata3[,4],newdata4[,4],
		newdata5[,4]),4))
	colnames(predictedP)=c("AincomeP","BincomeP","CincomeP","DincomeP")

	bin.pred=data.frame(cbind(newdata2[.1:3],predictedP))
	bin.pred
# This last table provides detailed information about the predicted
# probability of approval in relation to income and party membership.
# It will be of further help to plot these probabilities.

## Plotting Predicted Probabilities
	detach(bin.data)
	attach(bin.pred)
# Define the variables
	PartyA = bin.pred[,3]
	PartyB = bin.pred[,4]
	PartyC = bin.pred[,5]
	PartyD = bin.pred[,6]
# Plot the partyA data
	plot(PartyA,type="o", col="blue", ylim=c(0,1), lty=1,pch=18,lwd=2,
		axes=FALSE, ann=FALSE)
# Label the X axis and add titles
	axis(1, at=1:9, lab=c("20","30","40","50","60","70","80","90",
		"100"))	# X axis
	axis(2)		# Y axis
	box()			# Enclose plot
	title(xlab="Annual Income (*1,000)")
	title(ylab="P(Approve)")
	title(main="Approval by Income and Party Membership",font.main=4)
# Add	 the other three curves
	lines(PartyB, type="o", pch=19, lty=2, col="red",lwd=2)
	lines(PartyC, type="o", pch=20, lty=3, col="darkgreen",lwd=2)
	lines(PartyD, type="o", pch=21, lty=4, col="violet",lwd=2)
# Now add the legend
	legend(1,1, c("PartyA","PartyB","PartyC","PartyD"), cex=0.8, 
   		col=c("blue","red","darkgreen","violet"), pch=18:21, lty=1:4)

# Clearly, members of PartyA are estimated to have the highest probability
# of approval of the official at all income levels. Estimates for parties
# C and D are lowest, and relatively close together. The four corves diverge
# slightly as income increases.

	dev.off()

## Exercise 27.4 
# Get predictions as age varies from 10 to 80 in increments of 10
# for each party with income at its mean. Then plot the data.

## Exercise 27.5
# (a) Using the sib.data frame created at the start of Session 27, estimate
# all coefficients using all three variables (sex, age, agetoin).
# (b) Holding age at its mean, plot the predicted probabilities of sib
# as a function of agetoin for each sex.

## Predictions for Individual Cases:
	bin.dataP=cbind(bin.data,prob=round(predict(bin2.glm,type="response"),3))
	head(bin.dataP)

	detach(bin.pred)


### Logistic Regression using Mixed Effect Modeling
# We also can analyze the data using the lmer function from the lme4
# package (see Session 26). If not still loaded [use "search() to check],
	library(lme4)
# Then
	attach(bin.data)
# Now for the analysis
	bin.lmer=lmer(approve~income+age+factor(party)+(1|id),data=bin.data, 
		family="binomial")
	summary(bin.lmer)
# The coefficients are almost identical to those obtained using the glm 
# function. The lmer function provides additional meaasures of fit, as well 
# as correlations among the fixed effects.

# Clean up the work space
	detach(bin.data)
	rm(age,diff.dev,diff.df,income,bin.data,bin.glm,bin2.glm,bin.lmer,
		bin.pred,newdata1,newdata2,newdata3,newdata4,newdata5,party,
		bin.dataP,PartyA,PartyB,PartyC,PartyD,predictedP, sib.data)


### Multinomial Logistic Models: Nominal Predictors
# Binomial regression is a special case; more generally, we can
# have K response classes, and the scale may be ordinal (e.g., ratings) 
# or nominal (e.g., objects, such as brands that provide a choice). 
# We first consider dependent variables in which the response scale
# is nominal. We created a data set with six groups, 25 politically 
# conservative men (sex=m,label=C); 25 moderate men (m,M); and 25 
# liberal men (m,L). The remaining 3 groups of 25  each were women (sex = w) 
# Each individual indicates a preference for one of four politicians (A - D).
# The data are in the file "nominal.csv".

	Choices = read.csv(file.choose(),header=T,sep=",")
	head(Choices)
	Choices$choice=factor(Choices$choice)

## Summary and Analysis
# Let's look at the data. 
	table(Choices$choice)		 #Choice D has most preferences
	table(Choices$choice,Choices$sex)
	table(Choices$choice,Choices$label)
# Note the strong preference for D in group C, and the strong avoidance
# of D in groups M and L.

# If not already installed: install.packages("mlogit")
	library(mlogit)

# mlogit requires reshaping our data frame:
	Choices2=mlogit.data(Choices, varying=NULL, choice="choice", shape="wide")
	head(Choices2)
# alt indexes the alternative; choice indicates the chosen alternative

## A single binary predictor
# To introduce the model analysis, we ignore label and regress only on sex.
	ch0.mlogit=mlogit(choice~1|sex,data=Choices2,reflevel="A")
# A is the reference level, and the default if reflevel isn't specified.
		summary(ch0.mlogit)
# None of the coefficients reach the standard .05 significance level.

## Interpreting the Results of the mlogit Analysis
# Frequencies of alternatives = proportion of choices; for example, in 
# category A, we have 36/150 = .24

# Coefficients: The first three coefficients are the intercepts for choice
# categories B - D. Following these are the remaining 3 (sex) coefficients.
# These are the logs of the odds ratios (OR). For example, to obtain the
# OR(A,B) -- [P(B|w)/P(A|w)]/[P(B|m)/P(A|m)] -- we calculate exp(B:sexw)
# = exp(.80437), which equals 2.235. Note that the odds ratio from 
# the table of choice x sex is (26/17)/(13/19) = 2.235.
# From the OR(A,B) it is clear that women are more than twice
# as likely to prefer B to A than men are. 

# NOTE: In this one, binary predictor, example, the OR from the table and 
# exp(coefficient) are identical. With more predictors, this will not usually
# be the case. The best (maximum likelihood) estimate of the ORs will be
# obtained by exponentiating the coeffients.

## Now for the analysis of the complete data set. Because a preliminary
# analysis showed no evidence of a sex x label interaction, we assume
# an additive model.
	choice.mlogit=mlogit(choice~1|sex+label,data=Choices2,reflevel="A")
	summary(choice.mlogit)
# The interpretation of the sex coefficients are the same as in the
# one-factor regression, although the adjustment for label results in
# slightly different estimates. One obvious change is in the 
# estimate for C:sexw. Ignoring label (the ch0.mlogit summary), the
# estimate, -0.98739, is not significant at the .05 level. When label
$ is included in the analysis, the C:sexw estimate is -1.259988, and is
# significant with p = .04. The corresponding OR(A,C) is exp(-1.259988)
# = .284. The odds that women prefer C to A is a little over 1/4 of the 
# odds that men prefer C to A. 

# The label coefficients have similar interpretations. For example,
# consider the odds ratio,[P(D|labelL)/P(A|labelL)]/[P(D|labelC)/P(A|labelC)].
# The estimate = exp(-5.737149) = .0032. This is actually close to the
# result from the tabled frequencies.

## NOTE: The odds ratios can be obtained by
	exp(choice.mlogit$coefficients)[4:12]
# From the preceding results, we find that the estimated ORs are 
# approximately .01 or less for C:labelL, D:labelL,and D:labelM. These
# correspond to the three sources in the summary for which p < .001.

## Exercise 27.6
# Calculate the 6 observed OR for the three label groups. How do these
# compare with the estimates found from the model? How do they compare
# with estimates from a model that excludes the sex factor?

# Continuing with the output, the McFadden R^2 reflects the ratio in log
# likelihood for the null (intercept only) and full models; although not
# the same as the Pearson correlation, the higher the value. the better 
# the fit for the full model.

## Assumption of Independence of Irrelevant Alternatives (IIA)
# This assumption  underlies multinomial regression. It asserts that the
# ratio of probablilities for any two choices is independent of the 
# alternatives. If this is so, the results of an analysis using the full
# set of choices should not differ significantly from an analysis based
# on a subset of choices. The assumption is tested by the Hausman-McFadden 
# test. Here, two models are analyzed, the original one and a second one:
	choice2.mlogit=mlogit(choice~1|sex+label,data=Choices2,reflevel="A",
		alt.subset=c("A", "B"))
# Then the test is
	hmftest(choice.mlogit, choice2.mlogit)		#p = .967
# We can not reject the alternative hypothesis (that IIA is rejected).

## Exercise 27.7
# Compute the OR estimates for the choice 2 model and compare them with
# the corresponding ones for the full set of alternatives. Are they similar?

	rm(ch0.mlogit,choice.mlogit,choice2.mlogit,Choices,Choices2)


### Ordinal Logistic Regression
# There are several packdoses that contain functions for analyzing
# data when response classes are ordered; e.g. when responses are ratings.
# We will work with the rms package. Install it, if not already done.Then,
	library(rms)

# We have 75 men (sex = 0) and 75 women (sex=1) who receive either
# 1, 2, or 3 units/day of a new drug (dose = 1, 2, or 3), so that there
# are 6 groups of 25 patients each. At the end of 6 months, recovery
# is rated on a scale from 0 (no change) to 3 (excellent recovery).
# The data are in the file "ord.csv". 

	ord.data = read.csv(file.choose(),header=T,sep=",")
	head(ord.data)
	attach(ord.data)

# The next two lines store information about the predictors to be used later
# by the model. It is prerequisite to using the summary and anova functions.
	ord.dist = datadist(dose,sex)
	options(datadist="ord.dist")

# Look at the frequencies
	table(rating,dose)
	table(rating,sex)
	table(rating,dose,sex)
# These tables provide some feeling for the changes in odds. For example,
# when the dose = 1 unit/day, the odds favor an excellent recovery (rating
#  = 3) but the odds of that outcome decrease as the dosage increases. 

# A model including interaction is
	ord1.lrm=lrm(rating~dose*sex,data=ord.data)
	ord1.lrm
# The contributions of sex and dose*sex are clearly not significant. Therefore,
# simplify the model:
	ord.lrm=lrm(rating~dose,data=ord.data)
		ord.lrm

 ## Interpreting the Output
# In the output, following the statement of our formula, we have
# the frequencies of the responses [also given by "table(rating)"].

# In the next segment, of most interest is L.R., the chi-square test of
# our model based on the likelihood ratio. p is essentially 0,
# signifying that the model fit is a significant improvement
# over the null model. There are also several measures of the relationship
# between predicted probabilities and observed responses. C ("concordance")
# is one such measure, and is 0 if there is no relation, .5 if the relation
# is random, and 1 if it is perfect. Dxy = 2*(C-.5). The Brier score 
# is a measure of deviation of observed from predicted with lower scores
# being better. Gamma,tau-a, and R2 are measures of the relationship 
# between predicted probabilities and  observed responses. All of these 
# measures suggest more than a random relationship between predicted
# probabilities and observations.

# The first three coefficients are cut points to establish the bounds
# of the response classes, based on the assumption of a latent continuous
# variable underlying the ratings. Of more interest is the dose coefficient.
# The analysis rests on the assumption that the log odds, ln[O(j|x)], is 

#	ln[O(j|x)] = ln[P(rating<=j|dose=x)/P(rating>j|dose=x)] = a(j) - b*x

# where j is one of K (4 in the example) response categories, and
# b is the dose coefficient. One implication of the preceding equation
# is that if we were to plot ln[O(j|x)] for each value of j as a function
# of the dose, the lines wouild be parallel. A second implication
# is that the odds ratio (OR) is independent of j, To see why, 
# assume any two doses, x0 and x1. Then
#		OR=O(j|x1)/O(j|x0) and 
#   ln(OR)=ln[O(j|x1)]-ln[O(j|x0)]
# 		=[a(j)-b*x1]-[a(j)-b*x0]=-b*(x1-x0).

#  The model under consideration is called the proportional odds model
#  because the log of the odds is proportional to b and is independent
#  of j, the rating category.

# NOTE: The proportional odds model is one of a class of cumulative link
# models, so called because the odds are based on cumulative probabilities,
# unlike the nominal model considered earlier.

# The summary function provides the information in a different form:
	summary(ord.lrm)
# The Effect in the summary output is -3.41, the log of the OR,
# or (3-1)*b, where b is the dose coefficient in the output from
# ord.lrm. The Odds Ratio in the summary is .03, or
	exp(-3.41)
# This tells us that for a two unit increase in the dose, the ratio 
# of the odds of the j lowest rating categories versus the higher rating
# categories is .03. For a one unit increase in the dose, the OR is
	exp(-1.7039)				= .183
# If there are several predictors, we can simultanepusly convert all
# the coefficients to odds: 	exp(ord.lrm$coefficients)

## Predictions
# Still another way of understanding the data is to look at probabilities
# predicted by the model:
	dose=c(1,2,3)
	newdata1=data.frame(dose)
	newdata1$predicted=predict(ord.lrm,newdata=newdata1,type="fitted.ind")
	newdata1		# Predicted probabilities

## Exercise 27.8
# In the section on ginary logistic analysis, we plotted choice
# probabilities as a function of political party (curves for PartyA,
# B, C, and D) as a function of income. Here we want to plot rating 
# probabilities as a function of dose. We first need to establish levels
# of the ratings. Because the four rating columns in newdata1 are tresated
# as a single column [try "dim(newdata1)"], set y=reshape(newdata1[,2],3,4).
# Then, following the approach in the binary data section, plot each column
# of y as a function of dose in the same plot.

	rm(dose)	# Its presence causes a problem in the later test of
			# the proportional odds assumption

# Consistent with our understanding of the ord.lrm output, the probability 
# of a rating of 0 or 1 increases with dose whereas the probability 
# of higher ratings decrreases with dose. These probabilities may also 
# help us better understand what the logits represent as well as the 
# proportional odds assumption. We first calculate the log odds,
# using probabilities in newdata1.

# The log odds at the first cut point (rating = 0) when the dose = 1 is
# log(0.03719385/(1-0.03719385))= -3.253709
# With the same sutpoint but dose = 2,
# log(0.17511649/(1-0.17511649)) =  -1.549791
# The log of the odds ratio is  -3.253709 - (-1.549791) = -1.703918,
#  =b, the dose coefficient outputted by summary(ord.lrm)

# This is true for any cutpoint and any pair of doses 1 unit apart.
# For example, taking the seond cutpoint (rating = 1) and dose =2,
# log((0.17511649+ 0.35274037)/(1-(0.17511649+ 0.35274037))) = 0.1115429
# and for dose = 3 with the same cutpoint
# log((0.53845568+0.32156489)/(1-(0.53845568+0.32156489))) = 1.815461
# and the log odds again is  0.111542-1.815461 = -1.703919.

## Exercise 27.9
# Using the entries in newdata1, and doses 1 and 3, and cutpoints 1 and 3
# (ratins = 0 and 2), show that log(OR) = -3.408.

## Testing the Proportional Odds Model
# Bayen (2008, "Analyzing Linguistic Data", pp. 310-311) suggests the 
# following approach:
	model.lrm=lrm(rating~dose,x=T, y=T)
	resid(model.lrm,type='score.binary',pl=T)
# The output plots the mean of the partial binary residuals. Under the
# proportionality assumption, these should be close to zero. All the means
# are within confidence intervals containing 0,indicating no violation of
# the proportionality assumption.



### Ordinal Logistic Regression with Categorical Predictors
# Our previous analysis assumes a quantitative preictor.
# We can treat dose as a categorical variable (perhaps type of prescription):
	ord2.lrm=lrm(rating~as.factor(dose),data=ord.data)
	ord2.lrm					# The logits (log odds ratio)
# In this output, we have two lines of results for dose, with dose = 1
# as the referent point for dose=2 and dose=3:
#	dose=2 -2.0910 0.4144 -5.05  <0.0001 
#	dose=3 -3.4410 0.4639 -7.42  <0.0001 

# We can look at the odds ratios:
 	exp(ord2.lrm$coefficients)
# Relative to dose=1, the odds decline as ratings increase; this reflects
# the entries in the rating x dose table.

# We can get predicted rating probabilities for the various doses just as
# we did when dose was quantitative:
	dose=c(1,2,3)
	newdata2=data.frame(dose)
	newdata2$predicted=predict(ord2.lrm,newdata=newdata2,type="fitted.ind")
	newdata2

## Exercise 27.10
# Read the file "ordinal data categ.csv" and apply the lrm function. Note
# that dose is clearly categorical, labeled as "g1", "g2", and "g3". How do 
# the results of this output (ord3.lrm) compare to those of ord2.lrm?
 
	detach(ord.data)
	rm(dose,model.lrm,ord.lrm,ord1.lrm,ord2.lrm,ord.dist,newdata1,
		newdata2,ord.data)
